var string = {
    validate: function (value) {
        return typeof value === 'string';
    },
};
var number = {
    validate: function (value) {
        return typeof value === 'number';
    },
};
var boolean = {
    validate: function (value) {
        return typeof value === 'boolean';
    },
};
var nil = {
    validate: function (value) {
        return value === null;
    },
};
var unknown = {
    validate: function (value) {
        return true;
    },
};
function literal(str) {
    return {
        validate: function (value) {
            return value === str;
        },
    };
}
function any(validators) {
    return {
        validate: function (value) {
            return validators.some(function (validator) { return validator.validate(value); });
        },
    };
}
function nullable(validator) {
    return {
        validate: function (value) {
            return value === null || validator.validate(value);
        },
    };
}
function array(validator) {
    return {
        validate: function (value) {
            return Array.isArray(value) && value.every(function (it) { return validator.validate(it); });
        },
    };
}
function object(properties) {
    return {
        validate: function (value) {
            if (typeof value !== 'object' || value === null)
                return false;
            return Object.keys(properties).every(function (key) {
                var unescapedKey = unescapePropertyName(key);
                if (has(value, unescapedKey)) {
                    return properties[key].validate(value[unescapedKey]);
                }
                else {
                    return isOptionalProperty(key);
                }
            });
        },
    };
}
function isOptionalProperty(propertyName) {
    var _a;
    return ((_a = propertyName.match(/[^?]?(?:\?\?)*(\?)?$/)) === null || _a === void 0 ? void 0 : _a[1]) === '?';
}
function unescapePropertyName(propertyName) {
    var _a, _b;
    var match = propertyName.match(/^(.*?[^?]?)((?:\?\?)*)\??$/);
    if (match === null) {
        return propertyName;
    }
    var head = (_a = match[1]) !== null && _a !== void 0 ? _a : '';
    var questionMarks = (_b = match[2].replace(/\?\?/g, '?')) !== null && _b !== void 0 ? _b : '';
    return "" + head + questionMarks;
}
// https://github.com/microsoft/TypeScript/issues/21732#issuecomment-663994772
function has(target, property) {
    return property in target;
}

var j = {
    string: string,
    number: number,
    boolean: boolean,
    nil: nil,
    unknown: unknown,
    literal: literal,
    any: any,
    nullable: nullable,
    array: array,
    object: object,
};

export { j };
