export interface Validator<T> {
    validate(value: unknown): value is T;
}
export declare type JsonOf<T extends Validator<unknown>> = T extends Validator<infer U> ? U : never;
export declare const string: Validator<string>;
export declare const number: Validator<number>;
export declare const boolean: Validator<boolean>;
export declare const nil: Validator<null>;
export declare const unknown: Validator<unknown>;
export declare function literal<T extends string>(str: T): Validator<T>;
export declare function any<T extends Validator<unknown>>(validators: T[]): Validator<JsonOf<T>>;
export declare function nullable<T>(validator: Validator<T>): Validator<T | null>;
export declare function array<T extends Validator<unknown>>(validator: T): Validator<JsonOf<T>[]>;
export declare function object<T extends {
    [key: string]: Validator<unknown>;
}>(properties: T): Validator<ExpandRecursively<ObjectJsonOf<T>>>;
declare type FilterOptionalPropertyName<T> = T extends `${infer U}??` ? `${FilterOptionalPropertyName<U>}?` : T extends `${infer U}?` ? U : never;
declare type FilterRequiredPropertyName<T> = T extends `${infer U}??` ? `${FilterRequiredPropertyName<U>}?` : T extends `${infer U}?` ? never : T;
declare type ObjectJsonOf<T extends {
    [k: string]: Validator<unknown>;
}> = {
    [K in keyof T as FilterRequiredPropertyName<K>]-?: JsonOf<T[K]>;
} & {
    [K in keyof T as FilterOptionalPropertyName<K>]+?: JsonOf<T[K]>;
};
declare type ExpandRecursively<T> = T extends object ? T extends infer O ? {
    [K in keyof O]: ExpandRecursively<O[K]>;
} : never : T;
export declare function isOptionalProperty(propertyName: string): boolean;
export declare function unescapePropertyName(propertyName: string): string;
export {};
